function [t, f] = calfs(contx,omega,nu,alpha,qplus,coef,nbpts)
%function [t, f] = calfs(contx,omega,nu,alpha,qplus,coef,nbpts)
%
% Calcul de la fonction singuliere correspondant a l'exposant nu
%
% ENTREES
%  contx  : contexte de calcul pour chaque domaine (1=isotrope)
%  omega  : angles (en radians) definissant la geometrie des domaines
%  nu     : exposant de singularite
%  alpha  : racines du determinant du symbole M+
%  qplus  : vecteurs directeurs de la base correspondant a M+
%  coef   : coefficients de la combinaison lineaire definissant
%           la fonction singuliere (noyau de A(nu))
%  nbpts  : nombre de points de la courbe a calculer dans chaque domaine
%
% SORTIES
%  t      : angles d'evaluation en radians (vecteur ligne)
%  f      : valeurs de la fonction singuliere (matrice composee
%           d'un vecteur colonne pour chaque valeur de t)

dim   = size(alpha,1);
nbdom = size(alpha,2);
% Determination du nombre de points d'evaluation dans chaque domaine
fact = nbpts / (omega(nbdom+1)-omega(1));
nbp = round((omega(2:nbdom+1)-omega(1:nbdom))*fact);
%
l = sum(nbp);
f = zeros(dim,l);
t = zeros(1,l);
l = 0;
for dom=1:nbdom
 k = (dom-1)*dim;
 bj = 2*k;
 if contx(dom) == 1
%
%      Materiau isotrope
%
  for theta = linspace(omega(dom),omega(dom+1),nbp(dom))
   l = l+1;
   t(l) = theta;
   p = phiso(theta,nu,real(alpha(:,dom)),dim);
   for j=1:dim
    jpd = j+dim;
    f(:,l) = f(:,l) + coef(bj+j)*p(:,j) + coef(bj+jpd)*p(:,jpd);
   end
  end
 else
%
%      Materiau anisotrope
%
  for theta = linspace(omega(dom),omega(dom+1),nbp(dom))
   l = l+1;
   t(l) = theta;
   p = phian(theta,nu,alpha(:,dom),qplus(k+1:k+dim,:),dim);
   for j=1:dim
    jpd = j+dim;
    f(:,l) = f(:,l) + 0.5 * (coef(bj+j)    *(p(:,jpd)+p(:,j)) +...
                             coef(bj+jpd)*i*(p(:,jpd)-p(:,j)));
   end
  end
 end
end
