function p = phiso(theta,nu,cclame,dim)
%function p = phiso(theta,nu,cclame,dim)
%
% Calcul de la base des fonctions singulieres dans le cas isotrope
%
% ENTREES
%  theta  : coordonnee angulaire (abscisse d'evaluation en radians)
%  nu     : exposant de singularite
%  cclame : combinaisons des coefficients de Lame
%           (lambda + mu, lambda + 3*mu)
%  dim    : dimension de l'espace
%
% SORTIES
%  p : matrice des 2*dim vecteurs de la base

p = zeros(dim,2*dim);

val = theta * (nu-2);
psi = nu * cclame(1) * [-cos(val); sin(val)];

val = theta * nu;
p(1:2,1) = [cos(val); -sin(val)];
p(1:2,2) = [-p(2,1); p(1,1)];
p(1:2,3) = cclame(2) * [p(1,1); -p(2,1)] + psi;
psi = [psi(2); -psi(1)];
p(1:2,4) = cclame(2) * [p(2,1);  p(1,1)] + psi;

if dim == 3
 p(3,5) = p(1,2);
 p(3,6) = p(2,2);
end
