function tr1fs(dim,t,f,omega,nu,legy,hax,taicar,typcoo)
%function tr1fs(dim,t,f,omega,nu,legy,hax,taicar,typcoo)
%
% Trace d'une fonction singuliere angulaire associee a nu
% ou de la partie angulaire des contraintes correspondantes.
%
% ENTREES
%  dim    : dimension de l'espace
%  t      : angles d'evaluation en radians (vecteur)
%  f      : valeurs de la fonction en coordonnees cartesiennes (matrice com-
%           posee d'un vecteur colonne de longueur dim pour chaque valeur de t)
%  omega  : angles (en radians) definissant la geometrie des domaines
%  nu     : exposant de singularite
%  legy   : legende sur l'axe vertical du 1er graphique
%  hax    : pointeurs de gestion des graphiques
%  taicar : taille des caracteres (en points) pour les legendes
%  typcoo : type de coordonnees a utiliser pour tracer les deux premieres
%           composantes de f ; vaut
%             . 1 pour les coordonnees cartesiennes (valeur par defaut)
%             . 2 pour les coordonnees polaires
%

% Epaisseur des lignes du cadre (en points postscript)
eplica = 0.2;
% Conversion eventuelle en coordonnees polaires (seules les deux premieres
% composantes sont concernees)
if typcoo == 2
 xx = f(1,:);
 f(1,:) =  cos(t).*xx + sin(t).*f(2,:);
 f(2,:) = -sin(t).*xx + cos(t).*f(2,:);
end
% Conversion des abscisses en degres
t = t*180/pi;
omega = omega*180/pi;
% Normalisation des ordonnees
affin = 1 / max( max(max(abs(real(f)))), max(max(abs(imag(f)))) );
f = f*affin;
% Calcul des bornes de la fonction
ymin = -1; ymax =  1;
% ymin = min( min(min(real(f))), min(min(imag(f))) );
% ymax = max( max(max(real(f))), max(max(imag(f))) );
%
% Trace des courbes et du zero
coulax = get(0,'DefaultAxesXcolor');
nbdomp1 = length(omega);
for k=1:dim
 axes(hax(k))
 hp = plot([omega(1) omega(nbdomp1)],[0 0],':',...
           t,real(f(k,:)),'r', t,imag(f(k,:)),'b--');
 set(hp(1),'Color',coulax,'LineWidth',eplica)
 axis([omega(1) omega(nbdomp1) ymin ymax]);
 set(hax(k),'XTick',omega,'LineWidth',eplica);
% Trace des separateurs de domaines
 hold on
 yl=get(gca,'Ylim');
 for i=1:nbdomp1
  hq = plot([omega(i) omega(i)], yl);
  set(hq,'Color',coulax)
 end
% Modification de la taille des caracteres
 hr = hax(k);
 set(hr,'FontSize',taicar)
 set(get(hr,'xlabel'),'FontSize',taicar)
 set(get(hr,'ylabel'),'FontSize',taicar)
end
%
axes(hax(1))
ylabel(legy)
