function valnu = trfs(t,f,omega,nu,multi,sseprc,numfs,nomfic,titpar,typf,taicar)
%function valnu = trfs(t,f,omega,nu,multi,sseprc,numfs,nomfic,titpar,typf,taicar)
%
% Trace des fonctions singulieres angulaires associees a nu
% ou de la partie angulaire des contraintes correspondantes, dans le cas
% ou une figure est cree pour chaque exposant de singularite .
%
% ENTREES
%  t      : angles d'evaluation en radians (vecteur)
%  f      : valeurs de la fonction (matrice composee d'un vecteur
%           colonne pour chaque valeur de t)
%  omega  : angles (en radians) definissant la geometrie des domaines
%  nu     : exposant de singularite
%  multi  : multiplicite de nu
%  sseprc : seuil de separation des racines reelles et complexes
%  numfs  : numero de la fonction (compris entre 1 et multi)
%  nomfic : nom du fichier des donnees traitees
%  titpar : titre concernant le parametre de calcul. Peut etre vide.
%  typf   : typf(1) = type de fonction a tracer ; vaut
%             . 1 pour une fonction singuliere
%             . 2 pour les contraintes
%           typf(2) = type de coordonnees a utiliser pour tracer les deux
%                     premieres composantes de f ; vaut
%             . 1 pour les coordonnees cartesiennes (valeur par defaut)
%             . 2 pour les coordonnees polaires
%  taicar : taille des caracteres (en points) pour les legendes
%
% SORTIES
%  valnu  : chaine de caracteres correspondant a la valeur de nu
%

% Parametres regissant certains aspects du resultat :
nlix = 2;        % nombre de "lignes" de graphes (1er arg. de subplot)
		 % Pour que le dessin ait un aspect plus lisible dans
		 % une figure aux dimensions par defaut, lorsque la
		 % multiplicite vaut 1.
legtrait = notex('___ real  _ _ imag ');
if typf(1) == 1, legy = 'Sing. funct. '; else, legy = 'Stress funct. '; end
[v,d]=version;
if v(1) == '4', nutxt = 'nu'; else, nutxt = '\nu'; end
%
dim = size(f,1);
orig = dim*(numfs-1);
nli = max(nlix,multi); nco = dim;
hax = zeros(1,dim);
for k=1:dim
 hax(k) = subplot(nli,nco,orig+k);
end
tr1fs(dim,t,f,omega,nu,[legy,int2str(numfs)],hax,taicar,typf(2))
%
% Legende en x sur les graphiques inferieurs uniquement
if numfs == multi
 if typf(2) == 2
  labtyc = [' (radial)';' (angul.)';'         '];
 else
  labtyc = [' ';' ';' '];
 end
 for k=1:dim
  axes(hax(k))
  xlabel(['Comp. ',int2str(k),labtyc(k,:)])
 end
else
 set(hax(:),'XTick',[])
end
%
% On teste si l'exposant de singularite est reel (a sseprc pres),
% auquel cas la partie imaginaire de la fonction est nulle
% et n'est pas visible sur le graphique.
if (abs(imag(nu)) < sseprc)
 valnu = num2str(real(nu),7);
 leginfo = ' ';
else
 valnu = num2str(nu,7);
 leginfo = legtrait;
end
%
% Affichage des informations en tete du dessin
if numfs == 1
 leginfo = [leginfo,notex(nomfic)];
 titrenu = [nutxt,' = ',valnu];
 if dim == 2 & ~isempty(titpar), titrenu = [titpar,', ',titrenu]; end
 axes(hax(1)), title(titpar)
 axes(hax(dim-1)), title(titrenu)
 axes(hax(dim)), title(leginfo)
end
