************************************************************************
*  Calcul d'exposants de singularites en elasticite anisotrope.
*  Yvon Lafranche, Universite de Rennes 1.
*  Voir documentation a l'adresse :
*        http://perso.univ-rennes1.fr/yvon.lafranche/exsiel/
************************************************************************
************************************************************************
* AFFSYM : affectation de coefficients Aijkl
* APRIME : calcul d'un coefficient A'ijkl apres rotation
* CALCCP : calcul des coefficients cc' apres rotation
* MATCOA : formation de la matrice des coefficients Aijkl
* MATROT : formation de la matrice de rotation
* ROTPIJ : calcul de l'image d'un vecteur par une rotation
************************************************************************
      subroutine AFFSYM(cc,ind,A)
      real cc(*), A(3,3)
      integer ind(6)
*       -----------------------------------------------
*       Elasticite generale en dimension 3.
*       Affectation dans A (et symetrisation) des
*       9 coefficients Aijkl pour i et j fixes.
*       Arguments d'entree  : cc,ind
*       Arguments de sortie : A
*        cc  : coefficients des matrices de rigidite 3D
*        ind : tableau d'indices servant a l'adressage
*        A   : matrice des coef Aijkl pour i et j fixes
*       -----------------------------------------------
      A(1,1) = cc(ind(1))
      A(2,2) = cc(ind(2))
      A(3,3) = cc(ind(3))
      A(2,3) = cc(ind(4))
      A(1,3) = cc(ind(5))
      A(1,2) = cc(ind(6))
      A(3,2) = A(2,3)
      A(3,1) = A(1,3)
      A(2,1) = A(1,2)
      end
************************************************************************
      real function APRIME(ip,jp,kp,lp,A,R)
      integer ip,jp,kp,lp
      integer dim
      parameter (dim = 3)
      real A(dim,dim,dim,dim), R(dim,dim)
*  Variables locales
      integer i,j,k,l
      real si,sj,sk,sl
*       -----------------------------------------------
*       Elasticite generale en dimension 3.
*       Calcul du coefficient A'i'j'k'l' issu de la
*       rotation R, a partir des Aijkl.
*       Arguments d'entree  : ip,jp,kp,lp,A,R
*       Arguments de sortie : APRIME
*        ip,jp,kp,lp : indices i',j',k',l'
*        A   : matrice des coef Aijkl
*        R   : matrice de la rotation
*       -----------------------------------------------
      si = 0.
      do 40 i=1,dim
       sj = 0.
       do 30 j=1,dim
        sk = 0.
        do 20 k=1,dim
         sl = 0.
         do 10 l=1,dim
   10     sl = sl + R(l,lp)*A(l,k,j,i)
   20    sk = sk + R(k,kp)*sl
   30   sj = sj + R(j,jp)*sk
   40  si = si + R(i,ip)*sj
      APRIME = si
      end
************************************************************************
      subroutine CALCCP(cc,R,ccp)
      real cc(*), R(3,3), ccp(*)
*  Variables locales
      real A(3,3,3,3)
*  Point d'entree
      real APRIME
*       -----------------------------------------------------
*       Elasticite generale en dimension 3.
*       Calcul des 21 coefficients cc' issus de la rotation R
*       a partir des coefficients cc.
*       Arguments d'entree  : cc,R
*       Arguments de sortie : ccp
*       -----------------------------------------------------
      call MATCOA(cc,A)
      ccp(1) = APRIME(1,1,1,1,A,R)
      ccp(2) = APRIME(1,1,2,2,A,R)
      ccp(3) = APRIME(1,1,3,3,A,R)
      ccp(4) = APRIME(1,1,2,3,A,R)
      ccp(5) = APRIME(1,1,1,3,A,R)
      ccp(6) = APRIME(1,1,1,2,A,R)
      ccp(7) = APRIME(2,2,2,2,A,R)
      ccp(8) = APRIME(2,2,3,3,A,R)
      ccp(9) = APRIME(2,2,2,3,A,R)
      ccp(10) = APRIME(2,2,1,3,A,R)
      ccp(11) = APRIME(2,2,1,2,A,R)
      ccp(12) = APRIME(3,3,3,3,A,R)
      ccp(13) = APRIME(3,3,2,3,A,R)
      ccp(14) = APRIME(3,3,1,3,A,R)
      ccp(15) = APRIME(3,3,1,2,A,R)
      ccp(16) = APRIME(2,3,2,3,A,R)
      ccp(17) = APRIME(2,3,1,3,A,R)
      ccp(18) = APRIME(2,3,1,2,A,R)
      ccp(19) = APRIME(1,3,1,3,A,R)
      ccp(20) = APRIME(1,3,1,2,A,R)
      ccp(21) = APRIME(1,2,1,2,A,R)
      end
************************************************************************
      subroutine MATCOA(cc,A)
      real cc(*), A(3,3,3,3)
*  Variables locales
      integer ind(6,6)
      data ind /1, 2, 3, 4, 5, 6,  2, 7, 8, 9,10,11,  3, 8,12,13,14,15,
     &          4, 9,13,16,17,18,  5,10,14,17,19,20,  6,11,15,18,20,21/
*       ------------------------------------------------
*       Elasticite generale en dimension 3.
*       Formation de la matrice des coefficients Aijkl.
*       Le coefficient Aijkl est stocke dans A(l,k,j,i).
*       Arguments d'entree  : cc
*       Arguments de sortie : A
*        cc  : coefficients des matrices de rigidite 3D
*       ------------------------------------------------
      call AFFSYM(cc,ind(1,1),A(1,1,1,1))
      call AFFSYM(cc,ind(1,2),A(1,1,2,2))
      call AFFSYM(cc,ind(1,3),A(1,1,3,3))
      call AFFSYM(cc,ind(1,4),A(1,1,3,2))
      call AFFSYM(cc,ind(1,5),A(1,1,3,1))
      call AFFSYM(cc,ind(1,6),A(1,1,2,1))
*       Symetrisation
      call AFFSYM(cc,ind(1,4),A(1,1,2,3))
      call AFFSYM(cc,ind(1,5),A(1,1,1,3))
      call AFFSYM(cc,ind(1,6),A(1,1,1,2))
      end
************************************************************************
      subroutine MATROT(ang,axe,R)
      real ang(3), R(3,3)
      integer axe(3)
*  Variables locales
      real ct, st, t
      integer iper(3), i,j,k,l
*  Initialisation
      data iper /2,3,1/
*       ----------------------------------------------------------------
*       Calcul de la matrice de rotation R = Rot1.Rot2.Rot3 exprimee
*       dans la base (ei) de R3. R est la matrice de Rot3 o Rot2 o Rot1.
*       On definit fi = Rot1(ei), gi = Rot2(fi), hi = Rot3(gi) :
*
*               Rot1 / ej         Rot2 / fj         Rot3 / gj
*         (ei) -----------> (fi) -----------> (gi) -----------> (hi)
*
*       Pour i = 1,2,3, Roti est la rotation d'angle ang(i) (donne en
*       radians) autour de ej pour Rot1, fj pour Rot2, gj pour Rot3,
*       avec j = axe(i).
*       Par exemple, pour les angles d'Euler, compte tenu des
*       changements de bases :
*         psi (precession)      = ang(1), axe(1)=3 (/ e3 = z0)
*         theta (nutation)      = ang(2), axe(2)=1 (/ f1 = u)
*         phi (rotation propre) = ang(3), axe(3)=3 (/ g3 = z)
*       Soient M la matrice d'une application exprimee dans la base (ei)
*       et M' sa matrice exprimee dans la base (hi). On a M' = Rt M R,
*       ou Rt est la transposee de R.
*       Arguments d'entree  : ang,axe
*       Arguments de sortie : R
*       ----------------------------------------------------------------
*       Initialisation a Id
      do 20 j=1,3
       do 10 i=1,3
   10   R(i,j) = 0.
   20  R(j,j) = 1.
*       Rotations
      do 30 k=3,1,-1
       t = ang(k)
       ct = cos(t)
       st = sin(t)
       i = iper(axe(k))
       j = iper(i)
       do 30 l=1,3
   30   call ROTPIJ(ct,st,i,j,R(1,l))
      end
************************************************************************
      subroutine ROTPIJ(ct,st,i,j,U)
      real ct,st, U(*)
      integer i,j
*  Variables locales
      real t
*       ---------------------------------------------------------------
*       Calcul de l'image du vecteur U par la rotation definie par ct
*       et st (cos et sin de l'angle), dans le plan (ei,ej).
*       Pour conserver l'orientation dans Rn, donner le couple (i,j)
*       comme sous-ensemble d'une permutation circulaire de (1,2,...n).
*       Arguments d'entree  : ct,st,i,j,U
*       Arguments de sortie : U
*       ---------------------------------------------------------------
      t = U(i)*ct - U(j)*st
      U(j) = U(i)*st + U(j)*ct
      U(i) = t
      end
**************************** Fin du fichier ****************************
