************************************************************************
*  Calcul d'exposants de singularites en elasticite anisotrope.
*  Yvon Lafranche, Universite de Rennes 1.
*  Voir documentation a l'adresse :
*        http://perso.univ-rennes1.fr/yvon.lafranche/exsiel/
************************************************************************
************************************************************************
*                       Gestion des fichiers
* ATULOU : attribution d'une unite logique et ouverture d'un fichier
* DELANG : determination de la langue a utiliser
* EXTNFI : extraction du nom d'un fichier dans une arborescence UNIX
* GESFIC : ouverture de fichiers d'entree et de sortie
* GETULO : renvoi d'une unite logique valide
* KELNOM : analyse le nom d'un fichier ouvert
* LENOFI : lecture des noms de fichiers (interface avec l'environnement)
* OUVFIC : ouverture de fichiers de sortie
* ----------------------------------------------------------------------
*                Sous-programmes realisant les E/S
* CRTBMT : creation d'un tableau MATLAB
* DRCANB : numero du dernier caractere non blanc d'une chaine donnee
* ECRCOF : ecriture des coefficients de la matrice de rigidite
* ECRVAL : ecriture de valeurs entieres ou reelles
* GETENR : renvoi d'un enregistrement d'un fichier ascii
* LECVAL : lecture de valeurs entieres ou reelles
* PRCABL : numero du premier caractere blanc d'une chaine donnee
* PRCANB : numero du premier caractere non blanc d'une chaine donnee
* RCHMCL : recherche d'un mot cle dans un fichier
* SEPARR : impression d'une marque de separation dans un fichier
* ----------------------------------------------------------------------
*                        Gestion des messages
* IMPMES : impression d'un message en provenance d'un sous-programme
* INISTM : initialisation de la S.D. pour la gestion des messages
* MESSFR : liste des messages associes a chaque sous-prog., en francais
* MESSUK : liste des messages associes a chaque sous-prog., en anglais
************************************************************************
      subroutine ATULOU(nomfic,stat,unlo)
      character*(*) nomfic, stat
      integer unlo
*  Variables locales :
      logical ok
*       ------------------------------------------------------------
*       Attribution d'une unite logique et ouverture du fichier de
*       nom nomfic en mode sequentiel, formate et de statut stat.
*       Arguments d'entree  : nomfic, stat
*       Arguments de sortie : unlo
*        nomfic : nom du fichier a ouvrir
*        stat   : statut du fichier a ouvrir, typiquement 'OLD' pour
*                 un fichier de donnees, 'UNKNOWN' pour un fichier
*                 de resultat
*        unlo   : unite logique associee au fichier ouvert si aucun
*                 probleme n'a ete detecte, 0 sinon.
*       ------------------------------------------------------------
      call GETULO(unlo,ok)
      if (ok) then
       open(unit=unlo, file=nomfic, status=stat, err=10)
      else
       unlo = 0
      endif
      return
   10 unlo = 0
      end
************************************************************************
      subroutine DELANG(iue,codret)
      integer iue, codret
*  Variables locales
      character deblig*2, langue(2)
*  Initialisation
      data langue /'f', 'F'/
*       ---------------------------------------------------------------
*       Determination de la langue a utiliser en testant les premiers
*       caracteres du fichier de donnees associe a l'unite logique iue.
*       Reinitialisation de la structure des donnees des messages si la
*       langue choisie est differente de la langue par defaut qui est
*       l'anglais.
*       Le choix de la langue peut etre modifie de la maniere suivante :
*       si la premiere ligne du fichier de donnees est une ligne de
*       commentaire et si le deuxieme caractere est 'f' ou 'F', alors
*       la langue choisie est le francais.
*       Arguments d'entree  :
*        iue    : unite logique du fichier de donnees
*       Arguments de sortie :
*        codret : code de retour valant 0 si RAS, 1 si la
*                 memoire reservee pour la structure de
*                 donnees est insuffisante.
*       ---------------------------------------------------------------
      read(iue,'(a2)',err=10,end=10) deblig
   10 rewind(iue)
      if (deblig(1:1) .eq. '*' .and.
     &   (deblig(2:2).eq.langue(1) .or. deblig(2:2).eq.langue(2))) then
*      En francais
       call INISTM(1,codret)
      endif
      end
************************************************************************
      subroutine EXTNFI(fic,pc,dc)
      character*(*) fic
      integer pc, dc
*  Variables locales
      integer codret
*       -----------------------------------------------------------------
*       Extraction du nom d'un fichier dans une arborescence UNIX ou DOS.
*       Par exemple, si fic = '/projet/ssproj/fichier.txt',
*       pc et dc seront tels que fic(pc:dc) = 'fichier.txt'.
*       Arguments d'entree  : fic
*        fic    : nom absolu du fichier
*       Arguments de sortie : pc, dc
*        pc, dc : premier et dernier caractere du nom local du fichier
*       -----------------------------------------------------------------
      pc = 1
*       Recherche du dernier caractere non blanc
      do 10 dc=LEN(fic),1,-1
   10  if (fic(dc:dc) .ne. ' ') goto 20
      dc = 1
      goto 60
*       Recherche du separateur / ou \
   20 do 30 pc=dc,1,-1
   30  if (fic(pc:pc).eq.'/' .or. fic(pc:pc).eq.'\\') goto 40
      pc = 1
      goto 50
   40 pc = pc + 1
   50 return
   60 call IMPMES(0,'EXTNFI',1,' ',codret)
      end
************************************************************************
      subroutine GESFIC(intrfa,nbfix,unlo,nomfic,nbfiou)
      integer nbfix, unlo(nbfix), nbfiou
      character*(*) intrfa, nomfic(nbfix)
*  Variables locales :
      integer codret, nbfilu, nbfrou
      integer nbfres
      parameter (nbfres=2)
      character*4 ext(nbfres)
      data ext /'.res', '.exp'/
*       ------------------------------------------------------------
*       Ce sous-programme permet la liaison avec l'environnement
*       en effectuant :
*       . la lecture dans le fichier intrfa du nom du fichier de
*         donnees et le nom du repertoire ou seront stockes les
*         fichiers de resultat ; les noms sont attendus sous forme
*         de chemins absolus,
*       . l'attribution d'une unite logique a chacun des fichiers
*         a ouvrir,
*       . l'ouverture de ces fichiers en acces sequentiel formate.
*       Le nom de chaque fichier resultat est construit en ajoutant
*       une extension au nom du fichier de donnees. Deux fichiers
*       de resultat sont systematiquement produits par le programme ;
*       ils ont pour extension .res et .exp. Ce sous-programme n'ouvre
*       que ces deux fichiers de resultat ; les autres seront ouverts
*       si l'action demandee dans le fichier de donnees le necessite.
*       Arguments d'entree :
*        intrfa : nom du fichier contenant la liste des noms de
*                 fichiers a considerer
*        nbfix  : taille max des tableaux unlo et nomfic
*                 (nombre max de fichiers a ouvrir, doit etre >=3)
*       Arguments de sortie :
*        nbfiou : nombre de fichiers effectivement ouverts
*                 (doit valoir 3 en sortie de ce sous-programme)
*        unlo(i), i = 1,...nbfiou   : unites logiques associees
*                                     aux fichiers ouverts
*        nomfic(i), i = 1,...nbfiou : nom des fichiers ouverts
*       ------------------------------------------------------------
      nbfiou=0
      if (nbfix .lt. 3) then
       call IMPMES(0,'GESFIC',1,' ',codret)
       return
      endif
*            Lecture du fichier d'interface :
      call LENOFI(intrfa,nbfix,nomfic,nbfilu,codret)
      if (codret .ne. 0) then
* Ca demarre tres mal !
       call IMPMES(0,'GESFIC',2,' ',codret)
       return
      else if (nbfilu .ne. 2) then
* La structure du fichier interface est erronee
       call IMPMES(0,'GESFIC',3,' ',codret)
       return
      else
*       Attribution des unites logiques et ouverture des fichiers
*        1. cas du fichier de donnees
       nbfiou=1
       call ATULOU(nomfic(nbfiou),'OLD',unlo(nbfiou))
       if (unlo(nbfiou) .le. 0) then
        call IMPMES(0,'GESFIC',4,' '//nomfic(1),codret)
	nbfiou = nbfiou - 1
	return
       endif
*        2. cas des deux premiers fichiers resultat
       call OUVFIC(nomfic(1),nbfres,ext,unlo(2),nomfic(2),nbfrou)
       nbfiou = nbfiou + nbfrou
      endif
*
      end
************************************************************************
      subroutine GETULO(unlo,ok)
      integer unlo
      logical ok
* Variables locales
      integer i, ideb, ifin, codret
      logical exi, ouvert
      data ifin /150/
*       ----------------------------------------------------------
*       Cette fonction renvoie une unite logique valide dans unlo,
*       auquel cas ok est VRAI. La valeur proposee dans unlo est
*       prise en consideration. Si aucune unite logique n'est
*       trouvee, ok prend la valeur FAUX et unlo est indefini.
*       ATTENTION : deux appels consecutifs avec la meme valeur
*       d'entree pour unlo donnent le meme resultat, d'ou risque
*       d'erreur ===> utiliser l'unite logique pour ouvrir un
*       fichier immediatement apres son attribution.
*       Arguments d'entree :
*        unlo : unite logique proposee
*       Arguments de sortie :
*        unlo : unite logique valide si ok=VRAI
*        ok : VRAI attribution realisee, FAUX sinon.
*       ----------------------------------------------------------
      ideb = 7
      if (unlo.ge.ideb .and. unlo.le.ifin) ideb = unlo
      do 10 i=ideb,ifin
       inquire(unit=i,exist=exi,opened=ouvert,err=10)
       if (exi .and. .not.ouvert) goto 20
   10 continue
      ok = .false.
      call IMPMES(0,'GETULO',1,' ',codret)
      return
*
   20 ok = .true.
      unlo = i
      return
      end
************************************************************************
      subroutine KELNOM(unlo,nomrep,nomfic,ext,lgrep,lgfic,lgext)
      integer unlo,lgrep,lgfic,lgext
      character*(*) nomrep, nomfic, ext
*  Variables locales
      integer i, pc, dc, point
      character*200 nomabs
*       -----------------------------------------------------------
*       Eclate le nom absolu du fichier suppose ouvert sur l'unite
*       logique unlo en "nomrep/nomfic.ext". 
*       Arguments d'entree  :
*        unlo   : unite logique associee au fichier ouvert
*       Arguments de sortie :
*        nomrep : nom du repertoire contenant le fichier
*        nomfic : partie pricipale du nom du fichier
*        ext    : extension du nom du fichier
*                 (a partir du premier point exclu)
*        lgrep  : longueur du nom du repertoire
*        lgfic  : longueur du nom du fichier
*        lgext  : longueur du nom de l'extension
*       -----------------------------------------------------------
      inquire(unit=unlo,name=nomabs)
      call EXTNFI(nomabs,pc,dc)
      point = dc + 1
      do 10 i=pc,dc
       if (nomabs(i:i) .eq. '.') then
        point = i
        goto 20
       endif
   10 continue
   20 lgfic = min(dc,point-1) - pc + 1
      nomfic = nomabs(pc:pc+lgfic-1)
      if (point .lt. dc) then
       ext = nomabs(point+1:dc)
       lgext = dc-point
      else
       ext = ' '
       lgext = 0
      endif
      if (pc .le. 2) then
       nomrep = ' '
       lgrep = 0
      else
       nomrep = nomabs(:pc-2)
       lgrep = pc-2
      endif
      end
************************************************************************
      subroutine LENOFI(intrfa,nbfix,nomfic,nbfilu,codret)
      integer nbfix, nbfilu, codret
      character*(*) intrfa, nomfic(nbfix)
*  Communs
      character*200 repres
      common /OUTDIR/ repres
*  Variables locales
      integer i, unlo
      logical ok
*       --------------------------------------------------------------
*       Lecture des noms de fichier, figurant dans le fichier intrfa,
*       pour realiser l'interface avec l'environnement d'execution.
*       Au maximum nbfix noms de fichier sont lus.
*       Initialisation de la variable repres du commun OUTDIR (nom du
*       repertoire destine a contenir les resultats).
*       Arguments d'entree :
*        intrfa : nom du fichier contenant la liste des noms de
*                 fichiers a considerer
*        nbfix  : nombre max de fichiers susceptibles d'etre lus
*       Arguments de sortie :
*        nomfic : noms des fichiers
*        nbfilu : nombre de noms de fichiers effectivement lus
*        codret : code de retour valant 0 si RAS, 1 en cas d'erreur
*                 de lecture du fichier interface intrfa, 2 en
*                 cas de probleme d'allocation d'unite logique,
*                 3 si le fichier interface n'existe pas.
*       --------------------------------------------------------------
      do 5 i=1,LEN(repres)
    5  repres(i:i) = ' '
      codret = 0
      nbfilu = 0
      call GETULO(unlo,ok)
      if (ok) then
       open (unit=unlo,file=intrfa,status='old',err=40)
   10  nbfilu = nbfilu + 1
       if (nbfilu .gt. nbfix) goto 30
       read (unit=unlo,fmt='(a)',err=20,end=30) nomfic(nbfilu)
       goto 10
   20  call IMPMES(0,'LENOFI',1,'- '//intrfa,codret)
       codret = 1
   30  close (unlo)
       nbfilu = nbfilu - 1
       repres = nomfic(nbfilu)
      else
       call IMPMES(0,'LENOFI',2,'- '//intrfa,codret)
       codret = 2
      endif
      return
*
   40 call IMPMES(0,'LENOFI',3,'- '//intrfa,codret)
      codret = 3
      stop
      end
************************************************************************
      subroutine OUVFIC(ficdon,nbfic,ext,unlo,nomfic,nbfiou)
      integer nbfic, unlo(nbfic), nbfiou
      character*(*) ficdon, ext(nbfic), nomfic(nbfic)
*  Communs
      character*200 repres
      common /OUTDIR/ repres
*  Variables locales :
      integer i, pc,dc, lgrep, lgnfic, codret
*       ----------------------------------------------------------------
*       Ouverture des nbfic fichiers dont les extensions sont contenues
*       dans le tableau ext. Le nom de chaque fichier est construit a
*       partir du nom du fichier ficdon en lui ajoutant une de ces
*       extensions.
*       Le nom du repertoire contenant les fichier resultat est repres.
*       Arguments d'entree :
*        ficdon : nom du fichier de donnees
*        nbfic  : nombre de fichiers a ouvrir
*        ext    : tableau contenant les extensions des fichiers a ouvrir
*       Arguments de sortie :
*        unlo(i), i = 1,...nbfiou   : unites logiques associees
*                                     aux fichiers ouverts
*        nomfic(i), i = 1,...nbfiou : nom des fichiers ouverts
*        nbfiou : nombre de fichiers effectivement ouverts
*                 (doit valoir nbfic en sortie de ce sous-programme)
*       ----------------------------------------------------------------
      call EXTNFI(ficdon,pc,dc)
      lgrep = index(repres,' ')-1
      nbfiou=0
      do 10 i=1,nbfic
       nbfiou = nbfiou + 1
*       Construction du nom du fichier resultat
       nomfic(nbfiou) = repres(:lgrep)//ficdon(pc-1:dc)//ext(i)
       call ATULOU(nomfic(nbfiou),'UNKNOWN',unlo(nbfiou))
       if (unlo(nbfiou) .le. 0) then
        call IMPMES(0,'OUVFIC',1,' '//nomfic(nbfiou),codret)
        nbfiou = nbfiou - 1
       endif
*       write(*,*) 'Logical unit ',unlo(nbfiou), 'assigned to file '
       lgnfic = index(nomfic(nbfiou),' ')-1
       write(*,*)  nomfic(nbfiou)(:lgnfic), ' opened'
   10 continue
      write(*,*) ' '
      end
************************************************************************
************************************************************************
      subroutine CRTBMT(iu,ndec,nomtb,typ,nbi,nbj,nbk,tbi,tbr,tbc,
     &codret)
      integer iu,ndec,typ,nbi,nbj,nbk,tbi(nbi,nbj,nbk),codret
      real tbr(nbi,nbj,nbk)
      complex tbc(nbi,nbj,nbk)
      character*(*) nomtb
*  Variables locales
      integer i,j,k, decdef(0:2), ndecut
      character forma(0:2)*36
      data forma /'(..(1x,i11))',
     &            '(..(1x,g23.15e3))',
     &            '(..(1x,e23.15e3,''+ '',e23.15e3,''*i''))'/
      data decdef /11, 15, 15/
*       ---------------------------------------------------------------
*       Creation d'un tableau MATLAB.
*       Le tableau de nom nomtb et de type typ est cree au travers
*       d'un M-file, fichier associe a l'unite logique iu.
*       MATLAB ne manipule que des matrices ; un tableau a 3 dimensions
*       est donc remplace par une matrice (nbk*nbi, nbj).
*       Arguments d'entree  : iu,ndec,nomtb,nbi,nbj,tbr,tbc,typ
*       Arguments de sortie : codret
*        iu     : unite logique associee au M-file
*        ndec   : nombre de decimales a afficher pour les types real
*                 ou complex, largeur du champ pour le type integer.
*                 Si ndec <= 0, les valeurs par defaut decdef sont
*                 utilisees.
*        nomtb  : nom du tableau dans MATLAB
*        typ    : type des donnees (0 = integer, 1 = real, 2 = complex)
*        nbi,nbj,nbk: dimensions utiles des tableaux
*        tbi,tbr,tbc: tableaux de donnees a considerer (tbi si typ=0,
*                     tbr si typ=1, tbc si typ=2)
*        codret : code de retour valant
*                  0 si RAS
*                  1 en cas d'erreur sur la valeur de typ
*                  2 en cas d'erreur d'ecriture dans le fichier
*                  3 en cas d'erreur pour construire le format variable
*       ---------------------------------------------------------------
      codret=0
      if (typ.ge.0 .and. typ.le.2) then
       write(forma(typ)(2:3),'(i2)',err=40) nbj
       if (ndec .le. 0) then
        ndecut = decdef(typ)
       else
        ndecut = ndec
       endif
       write(iu,*,err=30) nomtb,' = ['
       if (typ.eq.0) then
        write(forma(typ)(9:10),'(i2)',err=40) ndecut
        do 5 k=1,nbk
        do 5 i=1,nbi
    5    write(iu,forma(typ),err=30) (tbi(i,j,k),j=1,nbj)
       else
        write(forma(typ)(9:13),'(i2,''.'',i2)',err=40) ndecut+8, ndecut
        if (typ.eq.1) then
         do 10 k=1,nbk
         do 10 i=1,nbi
   10     write(iu,forma(typ),err=30) (tbr(i,j,k),j=1,nbj)
        else
         write(forma(typ)(23:27),'(i2,''.'',i2)',err=40) ndecut+8,ndecut
         do 20 k=1,nbk
         do 20 i=1,nbi
   20     write(iu,forma(typ),err=30) (tbc(i,j,k),j=1,nbj)
        endif 
       endif 
       write(iu,*,err=30) ' ];'
      else
       call IMPMES(0,'CRTBMT',1,' ',codret)
       codret=1
      endif
      return
*
   30 call IMPMES(0,'CRTBMT',2,' ',codret)
      codret=2
      return
   40 call IMPMES(0,'CRTBMT',3,' ',codret)
      codret=3
      end
************************************************************************
      integer function DRCANB(ch,cd,cf)
      character*(*) ch
      integer cd, cf
*  Variables locales
      integer i
*       ------------------------------------------------
*       Renvoie le numero du dernier caractere non blanc
*       de la chaine ch entre les indices cd et cf.
*       Renvoie cf si aucun non blanc n'est detecte.
*       Arguments d'entree : ch,cd,cf
*       ------------------------------------------------
      if (cd .gt. cf) goto 20
      do 10 i=cf,cd,-1
   10  if (ch(i:i) .ne. ' ') goto 30
   20 i = cf
   30 DRCANB = i
      end
************************************************************************
      subroutine ECRCOF(unlo,dim,val,codret)
      integer unlo,dim,codret
      real val(*)
*  Variables locales
      integer i, k, ordpos(2:3), dec, larfor, ki,ks
      character forma*17
      data forma /'(txx,6(1x,g13.6))'/, larfor /14/, ordpos /3, 6/
*       ---------------------------------------------------
*       Ecriture des coefficients de la matrice de rigidite
*       (val(i),i=1,nb) dans le fichier sequentiel formate
*       associe a l'unite logique unlo.
*        codret : code de retour valant 0 si RAS, 1 sinon.
*        dim    : dimension de l'espace
*       Arguments d'entree  : unlo,dim,val
*       Arguments de sortie : codret
*       ---------------------------------------------------
      codret=0
      if (dim.eq.2 .or. dim.eq.3) then
       ki = 1
       ks = ordpos(dim)-1
       dec = 2 - larfor
       do 10 i=1,ordpos(dim)
        dec = dec + larfor
        write(forma(3:4),'(i2)') dec
        write(unlo,forma,err=30) (val(k),k=ki,ki+ks)
        ki = ki+ks+1
   10   ks = ks - 1
      else
       call IMPMES(0,'ECRCOF',1,' ',codret)
       codret=1
      endif
      return
*
   30 call IMPMES(0,'ECRCOF',2,' ',codret)
      codret=1
      end
************************************************************************
      subroutine ECRVAL(unlo,nb,typ,ival,val,codret)
      integer unlo,nb,typ,ival(nb),codret
      real val(nb)
*  Variables locales
      integer k
      character forma(2)*8
      data forma /'(40i2) ','(6g13.6)'/
*       ----------------------------------------------------------
*       Ecriture de nb valeurs entieres (ival(i),i=1,nb) ou reelles
*       (val(i),i=1,nb) selon la valeur de typ (1=entier, 2=reel),
*       dans le fichier sequentiel formate associe a l'unite
*       logique unlo.
*        codret : code de retour valant 0 si RAS, 1 sinon.
*       Arguments d'entree  : unlo,nb,typ,ival ou val
*       Arguments de sortie : codret
*       ----------------------------------------------------------
      codret=0
      if (typ.eq.1 .or. typ.eq.2) then
       if (typ.eq.1) then
        write(unlo,forma(typ),err=30) (ival(k),k=1,nb)
       else
        write(unlo,forma(typ),err=30) (val(k),k=1,nb)
       endif 
      else
       call IMPMES(0,'ECRVAL',1,' ',codret)
       codret=1
      endif
      return
*
   30 call IMPMES(0,'ECRVAL',2,' ',codret)
      codret=1
      end
************************************************************************
      subroutine GETENR(unlo,enr,codret)
      integer unlo, codret
      character*(*) enr
*      -----------------------------------------------------------
*      A partir de la position courante dans le fichier sequentiel
*      formate associe a l'unite logique unlo, lecture du premier
*      enregistrement enr ne commencant pas par '*'.
*      codret vaut 0 si RAS, 1 en cas d'erreur de lecture, 2 si la
*      fin du fichier est detectee.
*       Argument d'entree   : unlo
*       Arguments de sortie : enr, codret
*      -----------------------------------------------------------
      codret = 0
   10 read(unlo,'(a)',err=20,end=30) enr
      if (enr(1:1) .eq. '*') goto 10
      goto 40
   20 codret = 1
      goto 40
   30 codret = 2
   40 return
      end
************************************************************************
      subroutine LECVAL(unlo,nb,typ,ival,val,codret)
      integer unlo,nb,typ,ival(nb),codret
      real val(nb)
*  Points d'entree :
      integer PRCABL, PRCANB
*  Variables locales
      integer k, cd, cf, lgenr, nbcar
      character enr*90, forma(2)*8
      data forma /'(ixx)   ','(exx.0) '/
*       ----------------------------------------------------------
*       Lecture de nb valeurs entieres (ival(i),i=1,nb) ou reelles
*       (val(i),i=1,nb) selon la valeur de typ (1=entier, 2=reel)
*       dans le fichier sequentiel formate associe a l'unite
*       logique unlo.
*        codret : code de retour valant 0 si RAS, 1 en cas
*        d'erreur de lecture, 2 si la fin du fichier est detectee.
*       Arguments d'entree  : unlo,nb,typ
*       Arguments de sortie : ival ou val,codret
*       ----------------------------------------------------------
      if (typ.eq.1 .or. typ.eq.2) then
       lgenr = LEN(enr)
       k = 0
*        Lecture d'un enregistrement
   10  call GETENR(unlo,enr,codret)
       if (codret .eq. 2) goto 30
       if (codret .ne. 0) goto 40
*        Localisation de la valeur a lire : cd et cf determinent
*        le debut et la fin de la zone utile
       cf = 1
   20  cd = PRCANB(enr,cf,lgenr)
       cf = PRCABL(enr,cd,lgenr)
       nbcar = cf-cd
*        Generation du format si possible
       if (nbcar.gt.0) then
        write(forma(typ)(3:4),'(i2)') nbcar
       else
        goto 10
       endif
       k = k+1
       if (typ.eq.1) then
        read(enr(cd:cf-1),forma(typ),err=40) ival(k)
       else
        read(enr(cd:cf-1),forma(typ),err=40) val(k)
       endif 
       if (k.lt.nb) goto 20
      else
       call IMPMES(0,'LECVAL',1,' ',codret)
       codret=1
      endif
   30 return
*
   40 call IMPMES(0,'LECVAL',2,enr,codret)
      codret=1
      end
************************************************************************
      integer function PRCABL(ch,cd,cf)
      character*(*) ch
      integer cd, cf
*  Variables locales
      integer i
*       --------------------------------------------
*       Renvoie le numero du premier caractere blanc
*       de la chaine ch entre les indices cd et cf.
*       Renvoie cf+1 si aucun blanc n'est detecte.
*       Arguments d'entree : ch,cd,cf
*       --------------------------------------------
      if (cd .gt. cf) goto 20
      do 10 i=cd,cf
   10  if (ch(i:i) .eq. ' ') goto 30
   20 i = cf+1
   30 PRCABL = i
      end
************************************************************************
      integer function PRCANB(ch,cd,cf)
      character*(*) ch
      integer cd, cf
*  Variables locales
      integer i
*       ------------------------------------------------
*       Renvoie le numero du premier caractere non blanc
*       de la chaine ch entre les indices cd et cf.
*       Renvoie cf+1 si aucun non blanc n'est detecte.
*       Arguments d'entree : ch,cd,cf
*       ------------------------------------------------
      if (cd .gt. cf) goto 20
      do 10 i=cd,cf
   10  if (ch(i:i) .ne. ' ') goto 30
   20 i = cf+1
   30 PRCANB = i
      end
************************************************************************
      subroutine RCHMCL(unlo,motcle,codret)
      integer unlo, codret
      character*(*) motcle
*  Points d'entree :
      integer PRCANB
*  Variables locales
      integer i, cd, lgenr, lmc
      character enr*90
*       --------------------------------------------------------
*       Recherche du mot cle motcle a partir de la position
*       courante dans le fichier associe a l'unite logique unlo.
*       Arguments d'entree  : unlo,motcle
*       Arguments de sortie : codret
*       --------------------------------------------------------
      lgenr = LEN(enr)
      lmc   = LEN(motcle)
*
   10 call GETENR(unlo,enr,codret)
      if (codret .ne. 0) goto 30
      cd = PRCANB(enr,1,lgenr)
      do 20 i=1,lmc
       if (enr(cd:cd) .ne. motcle(i:i)) goto 10
       cd = cd+1
   20 continue
      return
*
   30 call IMPMES(0,'RCHMCL',1,'-'//motcle,codret)
      end
************************************************************************
      subroutine SEPARR(unlo,codret)
      integer unlo, codret
*       -----------------------------------------------------------
*        Impression d'une marque de separation dans le fichier unlo
*        codret : code de retour valant 0 si RAS, 1 sinon.
*       Argument d'entree  : unlo
*       Argument de sortie : codret
*       -----------------------------------------------------------
      codret = 0
      write(unlo,10,err=20)
   10 format(80('-'))
      return
   20 codret = 1
      end
************************************************************************
************************************************************************
      subroutine IMPMES(iu,module,num,mesupl,codret)
      integer iu, num, codret
      character*(*) module, mesupl
*  Points d'entree :
      integer DRCANB
*  Variables locales
      integer it, iad, adfin, lgtxt
      logical trouve
      character carsui
*  Communs
      include "lismes.inc"
*  Initialisation
      parameter (carsui = '-')
*       ---------------------------------------------------------------
*       Impression du message numero num relatif au module specifie sur
*       l'unite logique iu, avec eventuellement impression du message
*       supplementaire mesupl(2:...).
*       Si le numero du message est incompatible avec la structure de
*       donnees, le message imprime est vide.
*       Arguments d'entree  : iu,module,num,mesupl
*       Arguments de sortie : codret
*        iu     : numero de l'unite logique sur laquelle le message
*                 doit etre imprime. Si iu est <= 0, l'impression est
*                 faite sur la sortie standard.
*        module : nom du module emmetteur du message
*        num    : le numero du message a imprimer est abs(num) ;
*                 si num est positif, le message est imprime entre deux
*                 series d'etoiles, avec mention du module emetteur ;
*                 si num est negatif, le message est imprime tel quel.
*        mesupl : chaine de caracteres contenant le message supplemen-
*                 taire a imprimer.
*                 Le premier caractere sert au controle et n'est pas
*                 imprime. S'il vaut carsui, le message est imprime a
*                 la suite du message numero num sur la meme ligne ;
*                 sinon, il est imprime au dessous.
*        codret : code de retour valant 0 si RAS, 1 en cas d'erreur
*                 lors de l'impression.
*       ---------------------------------------------------------------
      codret = 0
*
*      Recherche de l'adresse du message a imprimer
*      nuderm est le numero du dernier module ayant imprime un message
*      (permet d'optimiser un peu du fait qu'un module imprime souvent
*      plusieurs messages consecutivement)
*
      iad = 0
      trouve = .true.
      do 10 it=nuderm,nbmod
   10  if (module .eq. nomodu(it)) goto 30
      do 20 it=1,nuderm-1
   20  if (module .eq. nomodu(it)) goto 30
      trouve = .false.
      goto 40
   30 nuderm = it
      if (num .eq. 0 .or. abs(num).gt.adad(it+1)-adad(it)-1) then
*       Numero du message hors des bornes (iad alors inutilise)
       trouve = .false.
      else
       iad = adad(it) + abs(num) - 1
      endif
*
*      Impression du message
*
   40 if (iu .gt. 0) then
*
*       Impression sur fichier
       if (num .gt. 0) write(iu,*,err=70) '*** ', module
       if (trouve) then
        adfin = ad(iad+1)-1
        do 50 it=ad(iad),adfin-1
   50    write(iu,*,err=70) textem(it)
        it = LEN(mesupl)
        if (it .gt. 1) then
         if (mesupl(1:1) .eq. carsui) then
	  lgtxt = LEN(textem(adfin))
          iad = DRCANB(textem(adfin),1,lgtxt)
          write(iu,*,err=70) textem(adfin)(1:iad)//mesupl(2:it)
         else
          write(iu,*,err=70) textem(adfin)
          write(iu,*,err=70) mesupl(2:it)
         endif
        else
         write(iu,*,err=70) textem(adfin)
        endif
       endif
       if (num .gt. 0) write(iu,*,err=70) '***'
*
      else
*
*       Impression sur la sortie standard : l'unite logique 6 n'est pas
*       affectee a cette fonction par tous les compilateurs, et selon le
*       contexte, peut etre deja associee a un fichier.
       if (num .gt. 0) write(*,*,err=70) '*** ', module
       if (trouve) then
        adfin = ad(iad+1)-1
        do 60 it=ad(iad),adfin-1
   60    write(*,*,err=70) textem(it)
        it = LEN(mesupl)
        if (it .gt. 1) then
         if (mesupl(1:1) .eq. carsui) then
	  lgtxt = LEN(textem(adfin))
          iad = DRCANB(textem(adfin),1,lgtxt)
          write(*,*,err=70) textem(adfin)(1:iad)//mesupl(2:it)
         else
          write(*,*,err=70) textem(adfin)
          write(*,*,err=70) mesupl(2:it)
         endif
        else
         write(*,*,err=70) textem(adfin)
        endif
       endif
       if (num .gt. 0) write(*,*,err=70) '***'
      endif
      return
*
   70 codret = 1
      end
************************************************************************
      subroutine INISTM(indlng,codret)
      integer indlng,codret
*  Variables locales
      integer lgt, ic, it, iad
*  Communs
      include "lismes.inc"
*       ----------------------------------------------------------------
*       Initialisation de la structure de donnees permettant la gestion
*       de l'affichage des messages.
*       Le texte des messages est memorises dans le tableau textem.
*       Les noms des modules emetteurs de messages sont memorises dans
*       le tableau nomodu.
*       nbmod est le nombre de modules emetteurs.
*       Soit i le rang d'un module dans le tableau nomodu.
*       Le premier message de ce module est a l'adresse ad(adad(i)) dans
*       le tableau textem, le suivant a l'adresse ad(adad(i)+1) et ainsi
*       de suite. Le nombre de messages concernant ce module est
*       adad(i+1)-adad(i)-1.
*       Arguments d'entree  : indlng
*       Arguments de sortie : codret
*        indlng : indice correspondant a la langue a utiliser
*                 0 = anglais, 1 = francais
*        codret : code de retour valant 0 si RAS, 1 si la
*                 memoire reservee pour la structure de
*                 donnees est insuffisante.
*       ----------------------------------------------------------------
*
*       Mise en place de la liste des messages.
*
      if (indlng .eq. 1) then
*      En francais
       do 20 it=1,nblfr
   20   textem(it) = textfr(it)
       nbltot = nblfr
      else
*      En anglais
       do 30 it=1,nbluk
   30   textem(it) = textuk(it)
       nbltot = nbluk
      endif
*
*       Initialisation de la structure de donnees
*
      codret = 0
      lgt = LEN(textem(1))
      nuderm = 1
      nbmod = 0
      iad = 0
      do 60 it=1,nbltot
*       Recherche d'un nom de module
       if (textem(it)(1:6) .eq. 'Module') then
        iad = iad + 1
        if (iad .gt. nbmesx) goto 90
        ad(iad) = it + 1
        nbmod = nbmod + 1
        if (nbmod .gt. nbmodx) goto 80
        nomodu(nbmod) = textem(it)(8:13)
        adad(nbmod) = iad
       else
*       Recherche de la fin d'un message
        do 40 ic=lgt,1,-1
   40    if (textem(it)(ic:ic) .eq. '`') goto 50
        goto 60
   50   iad = iad + 1
        if (iad .gt. nbmesx) goto 90
        ad(iad) = it + 1
        textem(it)(ic:ic) = ' '
       endif
   60 continue
      ic = nbmod + 1
      if (ic .gt. nbmodx) goto 80
      adad(ic) = iad + 1
CC       WRITE(*,*) 'nomodu'
CC       WRITE(*,'(11(1x,a))') (nomodu(it), it=1,nbmod)
CC       WRITE(*,*) 'adad'
CC       WRITE(*,*) (adad(it), it=1,nbmod+1)
CC       WRITE(*,*) 'ad'
CC       WRITE(*,*) (ad(it), it=adad(1),adad(nbmod+1)-1)
      return
*
*       Message d'erreurs !!
   80 write(*,*) '*** INISTM : memoire insuffisante (nbmodx).'
      write(*,*) '*** Arrive au module : ', nomodu(nbmod)
      codret = 1
      return
   90 write(*,*) '*** INISTM : memoire insuffisante (nbmesx).'
      write(*,*) '*** Arrive au module : ', nomodu(nbmod)
      codret = 1
      end
************************************************************************
      blockdata MESSFR
      include "dimefr.inc"
*      On decoupe en plusieurs tableaux car le compilateur SUN
*      ne permet pas plus de 98 lignes suites dans un data.
      character*63 mes1(lgfr1), mes2(lgfr2), mes3(lgfr3), mes4(lgfr4)
      character*63 mes5(lgfr5)
      common /txmefr/ mes1, mes2, mes3, mes4, mes5
*       -----------------------------------------------------------
*       Texte des messages en francais emis par chacun des modules.
*       La structure des tableaux est la suivante :
*        - nom du module
*        - message numero 1 termine par `
*        - ...
*        - message numero N termine par `
*       --> En cas de modification des tableaux mesi, ajuster la
*           valeur de la constante lgfri.
*       -----------------------------------------------------------
      data mes1 /
* RECRAC
     + 'Module QUADRI',
     1' ',
     /' ',
     /'      ================================================',
     /'      CALCUL AUTOMATIQUE DES EXPOSANTS DE SINGULARITES',
     /'      ================================================',
     /'`',
     2'Exploration du domaine :`',
     3'Domaine utilise        :`',
     4'Nombre de racines detectees a l''interieur :`',
     5'Analyse de la multiplicite. Nombre de points :`',
     6'Echec dans la recherche par quadrillage.`',
     7'Taille de la pile des racines insuffisante.`',
     8'Taille du tableau des abscisses insuffisante`',
     9'Erreur d''ecriture sur fichier resultat.`',
     + 'Module SUBCAD',
     1'Echec dans la subdivision.',
     /' Domaine, multiplicite, somme des racines :`',
     + 'Module SUICAL',
     1' ',
     /'--> La racine suivante est de multiplicite`',
     2' ',
     /'Approximation de l''exposant :`',
     3'Soit                        :`',
     4'    avec integrale de contour sur un cercle de rayon`',
     + 'Module TRCAD',
     1'Echec du zoom. Precision non atteinte.',
     /' Domaine, multiplicite, somme des racines :`',
     2'==> Interruption du calcul : taille min. de la maille atteinte.',
     /' taille courante, taille minimum :`',
     3'Precision non atteinte.',
     /' Domaine, multiplicite, somme des racines :`',
* CSTMAT
     + 'Module CALFS',
     1'%%%%%% Script MATLAB pour la representation %%%%%%',
     /'%%%%%%  graphique de fonctions singulieres  %%%%%%`',
     2'Erreur d''ecriture sur fichier.`',
     3'Erreur lors de la creation du M-file.`',
     + 'Module CALKER',
     1'Calcul du noyau de la matrice A(nu) pour`',
     2'Probleme ==> calcul de la fonction singuliere non fait.`',
     + 'Module KERANU',
     1' ',
     /'                 Caracterisation du noyau de A(nu)',
     /' ',
     /'nu = `',
     2'Multiplicite de nu : `',
     3' ',
     /'Le noyau de A(nu) est engendre par le(s) vecteur(s) :`',
     4' Vecteur`',
     5'           associe a la valeur singuliere `',
     6'Les valeurs singulieres suivantes ne sont pas correctes.`',
     7'Le noyau de A(nu) est de dimension `',
     8' --> Presence probable d''un terme logarithmique.`',
     9'Erreur sur la dimension du noyau de A(nu).',
     /'Multiplicite de nu, nombre de val. singulieres nulles :`',
     z'Valeur singuliere suivante : `'
     & /
      data mes2 /
* SINGUL
     + 'Module AFFQPM',
     1'Vecteur Q+ pour ALPHA`',
     + 'Module AFFRDS',
     1' ',
     /'Racines et modules du determinant du symbole :`',
     2'Detection d''une racine multiple.`',
     + 'Module CALALP',
     1'Echec dans le calcul des racines.`',
     2'Nombre de racines incorrect.`',
     3'Echec lors de l''ajustement de la localisation de ALPHA.',
     /'Numero de la racine ALPHA :`',
     + 'Module CALBAS',
     1' ',
     /' ',
     /'===============================================================',
     /'CALCUL DES RACINES ALPHA ET DES VECTEURS DE BASE DE LA SOLUTION',
     /'===============================================================',
     /'`',
     2' ',
     /'--- Domaine numero`',
     3'      Neant.`',
     + 'Module EXSIEL',
     1'Nombre de fichiers ouverts en debut d''execution incorrect.`',
     2' ',
     /'===== FIN DU CALCUL =====`',
     + 'Module RCMANU',
     1' ',
     /' ',
     /'      ================================================',
     /'      CALCUL INTERACTIF D''UN EXPOSANT DE SINGULARITE',
     /'      ================================================',
     /'`',
     2' Localisation interactive d''un exposant par calcul',
     /' d''integrales sur des cercles.`',
     3' Le nombre de points sur chaque cercle vaut :`',
     4'Donner une nouvelle valeur :`',
     5' Calcul d''integrales sur des cercles. Nombre de points :`',
     6'Centre et rayon ? ( Re(nu)  Im(nu)  r ), q pour quitter`',
     7' RESULTATS :',
     /'     Nu = exposant trouve, M = multiplicite,',
     /'     D = valeur du determinant, I = integrales correspondantes`',
     8' Calcul d''une estimation plus precise.`',
     9'Nouveau calcul (1), centre fixe + variation du rayon (2),',
     /'evaluation du determinant D (3), quitter (0) ?`',
     z' Centre conserve :`',
     1'Nouveau rayon ? ( <= 0 pour quitter)`',
     2'Valeur de nu ? ( Re(nu)  Im(nu) )`',
     3' Valeur du determinant :`',
     4' ',
     /' Exposant trouve :`',
     5' Avec integrale de contour sur un cercle de rayon`',
     6' ',
     /'--> Cette racine est de multiplicite`',
     + 'Module ROTFIB',
     1' ',
     /'                  ~~~~ Orientation des materiaux ~~~~',
     /'`',
     2'Pour le domaine numero`',
     3' Angle de rotation (en radians, puis en degres) :`',
     4' Coefficients Cij :`',
     + 'Module ROTREF',
     1' ',
     /'                  ~~~~ Rotation du referentiel ~~~~',
     /' ',
     /'Angle de rotation (en radians, puis en degres) :`',
     2'Coefficients Cij pour le domaine numero`',
     + 'Module SUIVAR',
     1' ',
     /'                  ~~~~ Suivi d''arete ~~~~',
     /' ',
     /'Parametre courant :`',
     2'Matrice de rotation donnee :`',
     3'Angles (en radians) delimitant chaque domaine :`',
     4'Coefficients Cij pour le domaine numero`',
     5'Donnees incompletes ou erronees.`',
     + 'Module VARCO',
     1' ',
     /'                  ~~~~ Variation d''un coefficient ~~~~',
     /' ',
     /'Coefficient courant :`',
     2'Donnees incompletes ou erronees.`',
     3'Matrice des coefficients non definie positive.`',
     4'Erreur detectee. Passage a la valeur suivante.`',
     5'Nouvelle matrice de rigidite :`',
     + 'Module VARGEO',
     1' ',
     /'                  ~~~~ Variation de la geometrie ~~~~',
     /' ',
     /'Position de la frontiere (en radians, puis en degres) :`',
     2'Angles (en radians) delimitant chaque domaine :`'
     & /
      data mes3 /
* LECVER
     + 'Module CMRISO',
     1'Erreur : condition Lambda > -Mu non remplie.`',
     + 'Module LIVDON',
     1'***********************************************************',
     /'*                                                         *',
     /'*   Calcul d''exposants de singularites dans le cadre de   *',
     /'*                  l''elasticite lineaire                  *',
     /'*                                                         *',
     /'***********************************************************',
     /' ',
     /' ',
     /'                ===========================',
     /'                PRISE EN COMPTE DES DONNEES',
     /'                ===========================',
     /'`',
     2'Erreur : DONNEES MANQUANTES !`',
     + 'Module LVACT',
     1'Action a realiser incorrecte. Valeur concernee :`',
     2'Erreur : l''action choisie n''est possible qu''en dimension 3.`',
     3'Action a realiser :`',
     z'   Calcul manuel`',
     1'   Calcul automatique seul`',
     2'   Calcul automatique avec rotation du referentiel`',
     3'   Calcul automatique avec variation de la geometrie`',
     4'   Calcul automatique avec orientation des materiaux`',
     5'   Calcul automatique avec suivi d''une arete courbe`',
     6'   Calcul automatique avec variation d''un coefficient`',
     z'`',
     1'   Calcul du noyau de A(nu)`',
     2'   Calcul du noyau et des fonction singulieres`',
     + 'Module LVANGL',
     1'Definition des angles incorrecte.`',
     2' --> omega(i-1) > omega(i) pour un i au moins`',
     3' --> omega(nbdom)-omega(0) > 360`',
     4'Angles delimitant chaque domaine',
     /'. en degres :`',
     5'. en radians :`',
     6'. en pi radians :`',
     + 'Module LVC2G',
     1'Coefficients Cij lus pour le domaine numero`',
     2'Erreur : contexte inderdit. Contexte et numero du domaine :`',
     + 'Module LVC3G',
     1'Coefficients Cij lus pour le domaine numero`',
     + 'Module LVCISO',
     1'Coefficients de Lame (Lambda, Mu) :`',
     2' . pour le domaine numero`',
     3'Numero du domaine concerne :`',
     + 'Module LVCLIM',
     1'Conditions aux limites incorrectes. Valeur concernee :`',
     2'Conditions aux limites incorrectes : incompatibles entre elles',
     /'ou avec les angles definissant les domaines.`',
     3'Conditions aux limites :`',
     1'   Dirichlet`',
     2'   Neumann`',
     3'   Mixte Dirichlet normal - Neumann tangentiel`',
     4'   Mixte Dirichlet tangentiel - Neumann normal`',
     5'   Transmission`'
     & /
*
      data mes4 /
     + 'Module LVCNTX',
     1'Contexte inconnu. Valeur concernee :`',
     2'Contextes donnes pour chaque domaine :`',
     1' Elasticite isotrope`',
     2' Elasticite orthotrope axisymetrique`',
     3' Elasticite orthotrope`',
     4' Elasticite generale`',
     + 'Module LVCOEF',
     1' ',
     /'~~~~~~',
     /' Matrices de rigidite correspondantes :',
     /'~~~~~~`',
     2'. pour le domaine numero`',
     3'Matrice non definie positive. Domaine numero :`',
     + 'Module LVDIM',
     1'Dimension de l''espace incorrecte. Valeur concernee :`',
     2'Dimension de l''espace :`',
     + 'Module LVDOM',
     1'Nombre de domaines incorrect.',
     /' Valeur concernee, puis valeur maximum :`',
     2'Nombre de domaines :`',
     + 'Module LVFRVA',
     1' Parametres relatifs a l''action "Variation de la geometrie".`',
     2'Numero de frontiere incorrect. Valeur concernee :`',
     3'Definition de la variation angulaire incoherente.',
     /' angle de depart, angle de fin, increment en radians :`',
     4' ',
     /'Numero de la frontiere variable :`',
     5'Definition de la variation angulaire.',
     /'Angle de depart, angle de fin, increment en degres :`',
     6'Soit en radians :`',
     + 'Module LVNUCO',
     1' Parametres relatifs a l''action "Variation d''un coefficient".',
     /' ',
     /'Numero du coefficient et domaine :`',
     2'Numero de domaine incorrect. Valeur concernee :`',
     3'Numero de coefficient incorrect. Valeur concernee :`',
     4'Numero interne correspondant :`',
     5'Variation de ce coefficient dans le domaine `',
     + 'Module LVRECT',
     1'Domaine de recherche des exposants (xmin, xmax, ymin, ymax) :`',
     2'Domaine de recherche des exposants incorrect.`',
     + 'Module LVRFIB',
     1' Parametres relatifs a l''action "Orientation des materiaux".`',
     2' ',
     /'Definition de la rotation pour le domaine numero`',
     3' Angle de depart, increment en degres :`',
     4' soit en radians :`',
     5' Numero de l''axe de rotation :`',
     6'Definition de la rotation incoherente.`',
     7' ',
     /'Nombre de pas a realiser et numero du domaine definissant',
     /'les abscisses du graphique :`',
     8'Nombre de pas ou numero du domaine incorrect.`',
     + 'Module LVROTA',
     1' Parametres relatifs a l''action "Rotation du referentiel".`',
     2'Definition de la rotation incoherente.`',
     3' ',
     /'Definition de la rotation',
     /' Angle de depart, angle de fin, increment en degres :`',
     4' soit en radians :`',
     5' Numero de l''axe de rotation :`',
     6'Numero d''axe incorrect.`'
     & /
      data mes5 /
* UTIES
     + 'Module EXTNFI',
     1'Erreur : chaine blanche.`',
     + 'Module GESFIC',
     1'Memoire insuffisante (nbfix)`',
     2'Erreur detectee en retour de LENOFI`',
     3'Le fichier de donnees interface n''a pas le format attendu.`',
     4'Erreur detectee apres tentative d''ouverture du fichier :`',
     + 'Module GETULO',
     1'Unite logique non attribuee.`',
     + 'Module LENOFI',
     1'Erreur de lecture dans le fichier d''interface :`',
     2'Probleme d''allocation d''unite logique au fichier interface :`',
     3'Fichier d''interface inexistant :`',
     + 'Module OUVFIC',
     1'Erreur detectee apres tentative d''ouverture du fichier :`',
     + 'Module CRTBMT',
     1'Erreur sur le parametre typ.`',
     2'Erreur d''ecriture dans le fichier.`',
     3'Erreur pour construire le format.`',
     + 'Module ECRCOF',
     1'Erreur sur le parametre dim.`',
     2'Erreur d''ecriture sur fichier.`',
     + 'Module ECRVAL',
     1'Erreur sur le parametre typ.`',
     2'Erreur d''ecriture sur fichier.`',
     + 'Module LECVAL',
     1'Erreur sur le parametre typ.`',
     2'Erreur de lecture ou de decodage dans l''enregistrement :`',
     + 'Module RCHMCL',
     1'Erreur de donnees. Mot cle non trouve : `'
     & /
      end
************************************************************************
      blockdata MESSUK
      include "dimeuk.inc"
*      On decoupe en plusieurs tableaux car le compilateur SUN
*      ne permet pas plus de 98 lignes suites dans un data.
      character*63 mes1(lguk1), mes2(lguk2), mes3(lguk3), mes4(lguk4)
      character*63 mes5(lguk5)
      common /txmeuk/ mes1, mes2, mes3, mes4, mes5
*       ----------------------------------------------------------
*       Texte des messages en anglais emis par chacun des modules.
*       La structure des tableaux est la suivante :
*        - nom du module
*        - message numero 1 termine par `
*        - ...
*        - message numero N termine par `
*       --> En cas de modification des tableaux mesi, ajuster la
*           valeur de la constante lguki.
*       ----------------------------------------------------------
      data mes1 /
* RECRAC
     + 'Module QUADRI',
     1' ',
     /' ',
     /'        ==============================================',
     /'        AUTOMATIC COMPUTATION OF SINGULARITY EXPONENTS',
     /'        ==============================================',
     /'`',
     2'Exploration of domain :`',
     3'Domain used           :`',
     4'Number of roots found inside :`',
     5'Analysis of multiplicity. Number of points :`',
     6'Cross-rule search failure.`',
     7'Root stack overflow.`',
     8'Abscissas array overflow`',
     9'Write error on output file.`',
     + 'Module SUBCAD',
     1'Subdivision failure.',
     /' Domain, multiplicity, sum of the roots :`',
     + 'Module SUICAL',
     1' ',
     /'--> The following root has multiplicity`',
     2' ',
     /'Approximation of the exponent :`',
     3'That is                       :`',
     4'        with an integral over a circle of radius`',
     + 'Module TRCAD',
     1'Zoom failure. Precision not reached.',
     /' Domain, multiplicity, sum of the roots :`',
     2'==> Computation break : minimum dimension of mesh reached.',
     /' current dimension, minimum dimension :`',
     3'Precision not reached.',
     /' Domain, multiplicity, sum of the roots :`',
* CSTMAT
     + 'Module CALFS',
     1'%%%%%%      MATLAB script for graphical      %%%%%%',
     /'%%%%%% representation of singular functions  %%%%%%`',
     2'Write error on file.`',
     3'Error while creating M-file.`',
     + 'Module CALKER',
     1'Computation of the kernel of the matrix A(nu) for`',
     2'Problem ==> computation of singular function not done.`',
     + 'Module KERANU',
     1' ',
     /'                 Caracterization of the kernel of A(nu)',
     /' ',
     /'nu = `',
     2'Multiplicity of nu : `',
     3' ',
     /'The kernel of A(nu) is spanned by the vector(s) :`',
     4' Vector`',
     5'          associated to the singular value `',
     6'The following singular values are not correct.`',
     7'The kernel of A(nu) has dimension `',
     8' --> A logarithmic term is probably existing.`',
     9'Error on the dimension of the kernel of A(nu).',
     /'Multiplicity of nu, number of zero singular values :`',
     z'Following singular value : `'
     & /
      data mes2 /
* SINGUL
     + 'Module AFFQPM',
     1'Vector Q+ for ALPHA`',
     + 'Module AFFRDS',
     1' ',
     /'Roots and moduli of the symbol determinant :`',
     2'Detection of a multiple root.`',
     + 'Module CALALP',
     1'Failure during roots computation.`',
     2'Incorrect number of roots.`',
     3'Failure during the adjustment of the localization of ALPHA.',
     /'Number of the root ALPHA :`',
     + 'Module CALBAS',
     1' ',
     /' ',
     /'   ========================================================',
     /'   COMPUTATION OF THE ROOTS ALPHA AND OF THE SOLUTION BASIS',
     /'   ========================================================',
     /'`',
     2' ',
     /'--- Domain number`',
     3'      None.`',
     + 'Module EXSIEL',
     1'Incorrect number of files opened at beginning of execution.`',
     2' ',
     /'===== END OF COMPUTATION =====`',
     + 'Module RCMANU',
     1' ',
     /' ',
     /'       =================================================',
     /'       INTERACTIVE COMPUTATION OF A SINGULARITY EXPONENT',
     /'       =================================================',
     /'`',
     2' Interactive localisation of a exponent by computation',
     /' of integrals over circles.`',
     3' The number of points on each circle is :`',
     4'Give a new number :`',
     5' Computation of integrals over circles. Number of points :`',
     6'Center and ray ? ( Re(nu)  Im(nu)  r ), q to quit`',
     7' RESULTS :',
     /'     Nu = exponent found, M = multiplicity,',
     /'     D = determinant value, I = corresponding integrals`',
     8' Computation of a more accurate estimation.`',
     9'New calculus (1), center fixed + variation of the ray (2),',
     /'valuation of the determinant D (3), quit (0) ?`',
     z' Center kept :`',
     1'New ray ? ( <= 0 to quit)`',
     2'Value of nu ? ( Re(nu)  Im(nu) )`',
     3' Value of the determinant :`',
     4' ',
     /' Exponent found :`',
     5' With an integral over a circle of radius`',
     6' ',
     /'--> This root has multiplicity`',
     + 'Module ROTFIB',
     1' ',
     /'                  ~~~~ Orientation of materials ~~~~',
     /'`',
     2'For the domain number`',
     3' Rotation angle (in radians, then in degrees) :`',
     4' Cij coefficients :`',
     + 'Module ROTREF',
     1' ',
     /'                  ~~~~ Rotation of axes ~~~~',
     /' ',
     /' Rotation angle (in radians, then in degrees) :`',
     2'Cij coefficients for the domain number`',
     + 'Module SUIVAR',
     1' ',
     /'                  ~~~~ Edge tracking ~~~~',
     /' ',
     /'Current parameter :`',
     2'Given rotation matrix :`',
     3'Angles (in radians) delimiting each domain :`',
     4'Cij coefficients for the domain number`',
     5'Incomplete or erroneous data.`',
     + 'Module VARCO',
     1' ',
     /'                  ~~~~ Variation of a coefficient ~~~~',
     /' ',
     /'Current coefficient :`',
     2'Incomplete or erroneous data.`',
     3'Matrix of coefficients not positive definite.`',
     4'Error detected. Step to next value`',
     5'New rigidity matrix:`',
     + 'Module VARGEO',
     1' ',
     /'                  ~~~~ Variation of the geometry ~~~~',
     /' ',
     /'Position of the boundary (in radians, then in degrees) :`',
     2'Angles (in radians) delimiting each domain :`'
     & /
      data mes3 /
* LECVER
     + 'Module CMRISO',
     1'Error : condition Lambda > -Mu not verified.`',
     + 'Module LIVDON',
     1'***************************************************************',
     /'*                                                             *',
     /'*  Computation of singularity exponents in linear elasticity  *',
     /'*                                                             *',
     /'***************************************************************',
     /' ',
     /' ',
     /'                  ========================',
     /'                  TAKING DATA INTO ACCOUNT',
     /'                  ========================',
     /'`',
     2'Error : MISSING DATA !`',
     + 'Module LVACT',
     1'Incorrect action to be done. Concerned value :`',
     2'Error : the requested action is only available in dimension 3.`',
     3'Action to be done :`',
     z'   Manual computation`',
     1'   Automatic computation only`',
     2'   Automatic computation with rotation of axes`',
     3'   Automatic computation with variation of the geometry`',
     4'   Automatic computation with orientation of the materials`',
     5'   Automatic computation with curved edge tracking`',
     6'   Automatic computation with variation of a coefficient`',
     z'`',
     1'   Computation of the kernel of A(nu)`',
     2'   Computation of the kernel and the singular functions`',
     + 'Module LVANGL',
     1'Incorrect definition of the angles.`',
     2' --> omega(i-1) > omega(i) for at least one i`',
     3' --> omega(nbdom)-omega(0) > 360`',
     4'Angles delimiting each domain',
     /'. in degrees :`',
     5'. in radians :`',
     6'. in pi radians :`',
     + 'Module LVC2G',
     1'Cij coefficients read for the domain number`',
     2'Error : forbidden context. Context and domain number :`',
     + 'Module LVC3G',
     1'Cij coefficients read for the domain number`',
     + 'Module LVCISO',
     1'Lame coefficients (Lambda, Mu) :`',
     2' . for the domain number`',
     3'Number of concerned domain :`',
     + 'Module LVCLIM',
     1'Incorrect boundary conditions. Concerned value :`',
     2'Incorrect boundary conditions : incompatible among themselves',
     /'or with angles defining the domains.`',
     3'Boundary conditions :`',
     1'   Dirichlet`',
     2'   Neumann`',
     3'   Mixed normal Dirichlet - tangent Neumann`',
     4'   Mixed tangent Dirichlet - normal Neumann`',
     5'   Transmission`'
     & /
*
      data mes4 /
     + 'Module LVCNTX',
     1'Unknown context. Concerned value :`',
     2'Given contexts for each domain :`',
     1' Isotropic elasticity`',
     2' Axisymetric orthotropic elasticity`',
     3' Orthotropic elasticity`',
     4' General elasticity`',
     + 'Module LVCOEF',
     1' ',
     /'~~~~~~',
     /' Corresponding rigidity matrices :',
     /'~~~~~~`',
     2'. for the domain number`',
     3'Non positive definite matrix. Domain number :`',
     + 'Module LVDIM',
     1'Incorrect dimension of the space. Concerned value :`',
     2'Dimension of the space :`',
     + 'Module LVDOM',
     1'Incorrect number of domains.',
     /' Concerned value, then maximum value :`',
     2'Number of domains :`',
     + 'Module LVFRVA',
     1' Parameters related to the action "Variation of the geometry".`',
     2'Incorrect boundary number. Concerned value :`',
     3'Inconsistent definition of the angular variation.',
     /' starting angle, end angle, step in radians :`',
     4' ',
     /'Number of the variable boundary`',
     5'Definition of the angular variation.',
     /'Starting angle, end angle, step in degrees :`',
     6'That is in radians :`',
     + 'Module LVNUCO',
     1' Parameters related to the action "Variation of a coefficient".',
     /' ',
     /'Coefficient number and domain:`',
     2'Incorrect domain number. Concerned value :`',
     3'Incorrect coefficient number. Concerned value :`',
     4'Corresponding internal index :`',
     5'Variation of this coefficient in domain `',
     + 'Module LVRECT',
     1'Search domain for the exponents (xmin, xmax, ymin, ymax) :`',
     2'Incorrect exponent search domain.`',
     + 'Module LVRFIB',
     1' Parameters related to action "Orientation of the materials".`',
     2' ',
     /'Definition of the rotation for the domain number`',
     3' Starting angle, step in degrees :`',
     4' that is in radians :`',
     5' Number of the rotation axis :`',
     6'Inconsistent definition of the rotation.`',
     7' ',
     /'Number of steps to make and number of the domain defining',
     /'the abscissas of the graph :`',
     8'Incorrect number of steps or domain number.`',
     + 'Module LVROTA',
     1' Parameters related to the action "Rotation of axes".`',
     2'Inconsistent definition of the rotation.`',
     3' ',
     /'Definition of the rotation',
     /' Starting angle, end angle, step in degrees :`',
     4' that is in radians :`',
     5' Number of the rotation axis :`',
     6'Incorrect number of the axis.`'
     & /
      data mes5 /
* UTIES
     + 'Module EXTNFI',
     1'Error : white string.`',
     + 'Module GESFIC',
     1'Insufficient memory (nbfix)`',
     2'Error detected after exit from LENOFI`',
     3'The interface data file has an unexpected format.`',
     4'Error detected after trying to open the file:`',
     + 'Module GETULO',
     1'Logical unit not assigned.`',
     + 'Module LENOFI',
     1'Read error on interface file:`',
     2'Problem in assigning a logical unit to interface file:`',
     3'Interface file not found:`',
     + 'Module OUVFIC',
     1'Error detected after trying to open the file:`',
     + 'Module CRTBMT',
     1'Error on the argument typ.`',
     2'Write error on file.`',
     3'Error while building the format.`',
     + 'Module ECRCOF',
     1'Error on the argument dim.`',
     2'Write error on file.`',
     + 'Module ECRVAL',
     1'Error on the argument typ.`',
     2'Write error on file.`',
     + 'Module LECVAL',
     1'Error on the argument typ.`',
     2'Read or decode error in the record :`',
     + 'Module RCHMCL',
     1'Data error. Keyword not found : `'
     & /
      end
**************************** Fin du fichier ****************************
